import * as _nuxt_schema from '@nuxt/schema';
import { Theme, Lang } from 'shiki-es';

declare type MountOptions = {
    name: string;
    prefix?: string;
    driver: 'fs' | 'http' | string;
    [options: string]: any;
};
interface ModuleOptions {
    /**
     * Base route that will be used for content api
     *
     * @default '_content'
     */
    base: string;
    /**
     * Disable content watcher and hot content reload.
     * Note: Watcher is a development feature and will not includes in the production.
     *
     * @default true
     */
    watch: boolean;
    /**
     * Contents can located in multiple places, in multiple directories or even in remote git repositories.
     * Using sources option you can tell Content module where to look for contents.
     *
     * @default ['content']
     */
    sources: Array<string | MountOptions>;
    /**
     * List of ignore pattern that will be used for excluding content from parsing and rendering.
     *
     * @default ['\\.', '-']
     */
    ignores: Array<string>;
    /**
     * Content module uses `remark` and `rehype` under the hood to compile markdown files.
     * You can modify this options to control its behavior.
     */
    markdown: {
        /**
         * Whether MDC syntax should be supported or not.
         *
         * @default true
         */
        mdc?: boolean;
        /**
         * Control behavior of Table of Contents generation
         */
        toc?: {
            /**
             * Maximum heading depth that includes in the table of contents.
             *
             * @default 2
             */
            depth?: number;
            /**
             * Maximum depth of nested tags to search for heading.
             *
             * @default 2
             */
            searchDepth?: number;
        };
        /**
         * Tags will be used to replace markdown components and render custom components instead of default ones.
         *
         * @default {}
         */
        tags?: Record<string, string>;
        /**
         * Register custom remark plugin to provide new feature into your markdown contents.
         * Checkout: https://github.com/remarkjs/remark/blob/main/doc/plugins.md
         *
         * @default []
         */
        remarkPlugins?: Array<string | [string, any]>;
        /**
         * Register custom remark plugin to provide new feature into your markdown contents.
         * Checkout: https://github.com/rehypejs/rehype/blob/main/doc/plugins.md
         *
         * @default []
         */
        rehypePlugins?: Array<string | [string, any]>;
    };
    /**
     * Content module uses `shiki` to highlight code blocks.
     * You can configure Shiki options to control its behavior.
     */
    highlight: false | {
        /**
         * Default theme that will be used for highlighting code blocks.
         */
        theme: Theme;
        /**
         * Preloaded languages that will be available for highlighting code blocks.
         */
        preload: Lang[];
    };
    /**
     * Options for yaml parser.
     *
     * @default {}
     */
    yaml: false | Record<string, any>;
    /**
     * Enable/Disable navigation.
     *
     * @defaul true
     */
    navigation: boolean;
    /**
     * List of locale codes.
     * This codes will be used to detect contents locale.
     *
     * @default []
     */
    locales: Array<string>;
    /**
     * Default locale for top level contents.
     *
     * @default undefined
     */
    defaultLocale: string;
}
interface ContentContext extends ModuleOptions {
    base: Readonly<string>;
    transformers: Array<string>;
}
interface ModuleHooks {
    'content:context'(ctx: ContentContext): void;
}
declare const _default: _nuxt_schema.NuxtModule<ModuleOptions>;

interface ModulePublicRuntimeConfig {
    tags: Record<string, string>;
    base: string;
    wsUrl?: string;
    highlight: ModuleOptions['highlight'];
}
interface ModulePrivateRuntimeConfig {
}
declare module '@nuxt/schema' {
    interface ConfigSchema {
        publicRuntimeConfig?: {
            content?: ModulePublicRuntimeConfig;
        };
        privateRuntimeConfig?: {
            content?: ModulePrivateRuntimeConfig & ContentContext;
        };
    }
}

export { ModuleHooks, ModuleOptions, MountOptions, _default as default };
