import { useLogger, resolveModule, defineNuxtModule, createResolver, addPlugin, templateUtils, addAutoImport, addComponentsDir } from '@nuxt/kit';
import defu from 'defu';
import { createStorage } from 'unstorage';
import { resolve, join } from 'pathe';
import { listen } from 'listhen';
import { debounce } from 'perfect-debounce';
import fsDriver from 'unstorage/drivers/fs';
import httpDriver from 'unstorage/drivers/http';
import { WebSocketServer } from 'ws';

// -- Unbuild CommonJS Shims --
import __cjs_url__ from 'url';
import __cjs_path__ from 'path';
import __cjs_mod__ from 'module';
const __filename = __cjs_url__.fileURLToPath(import.meta.url);
const __dirname = __cjs_path__.dirname(__filename);
const require = __cjs_mod__.createRequire(import.meta.url);


const name = "@nuxt/content-edge";
const version = "2.0.0-27509284.c41d0ab";

const MOUNT_PREFIX = "content:source:";
const PROSE_TAGS = [
  "p",
  "a",
  "blockquote",
  "code-inline",
  "code",
  "em",
  "h1",
  "h2",
  "h3",
  "h4",
  "h5",
  "h6",
  "hr",
  "img",
  "ul",
  "ol",
  "li",
  "strong",
  "table",
  "thead",
  "tbody",
  "td",
  "th",
  "tr"
];
function getMountDriver(mount) {
  if (mount.driver === "fs") {
    return fsDriver(mount);
  }
  if (mount.driver === "http") {
    return httpDriver(mount);
  }
  try {
    return require(mount.driver).default(mount);
  } catch (e) {
    console.error("Couldn't load driver", mount.driver);
  }
}
function useContentMounts(nuxt, storages) {
  const key = (path, prefix = "") => `${MOUNT_PREFIX}${path.replace(/[/:]/g, "_")}${prefix.replace(/\//g, ":")}`;
  return storages.reduce((mounts, storage) => {
    if (typeof storage === "string") {
      mounts[key(storage)] = {
        name: storage,
        driver: "fs",
        prefix: "",
        base: resolve(nuxt.options.srcDir, storage)
      };
    }
    if (typeof storage === "object") {
      mounts[key(storage.name, storage.prefix)] = storage;
    }
    return mounts;
  }, {});
}
function createWebSocket() {
  const wss = new WebSocketServer({ noServer: true });
  const serve = (req, socket = req.socket, head = "") => wss.handleUpgrade(req, socket, head, (client) => wss.emit("connection", client, req));
  const broadcast = (data) => {
    data = JSON.stringify(data);
    for (const client of wss.clients) {
      try {
        client.send(data);
      } catch (err) {
      }
    }
  };
  return {
    serve,
    broadcast,
    close: () => new Promise((resolve2) => wss.close(resolve2))
  };
}
function processMarkdownOptions(nuxt, options) {
  options.rehypePlugins = (options.rehypePlugins || []).map(resolveMarkdownPlugin).filter(Boolean);
  options.remarkPlugins = (options.remarkPlugins || []).map(resolveMarkdownPlugin).filter(Boolean);
  return options;
  function resolveMarkdownPlugin(plugin) {
    if (typeof plugin === "string") {
      plugin = [plugin, {}];
    }
    if (!Array.isArray(plugin)) {
      useLogger("@nuxt/content").warn("Plugin silently ignored:", plugin.name || plugin);
      return;
    }
    return [resolveModule(plugin[0], { paths: nuxt.options.srcDir }), plugin[1]];
  }
}

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "content",
    compatibility: {
      nuxt: "^3.0.0",
      bridge: true
    }
  },
  defaults: {
    base: "_content",
    watch: true,
    sources: ["content"],
    ignores: ["\\.", "-"],
    locales: [],
    defaultLocale: void 0,
    highlight: false,
    markdown: {
      tags: Object.fromEntries(PROSE_TAGS.map((t) => [t, `prose-${t}`]))
    },
    yaml: {},
    navigation: true
  },
  async setup(options, nuxt) {
    const { resolve } = createResolver(import.meta.url);
    const resolveRuntimeModule = (path) => resolveModule(path, { paths: resolve("./runtime") });
    const logger = useLogger("@nuxt/content");
    const contentContext = {
      transformers: [
        resolveRuntimeModule("./server/transformers/markdown"),
        resolveRuntimeModule("./server/transformers/yaml"),
        resolveRuntimeModule("./server/transformers/path-meta")
      ],
      ...options
    };
    if (nuxt.options.vite !== false) {
      nuxt.options.vite = defu(nuxt.options.vite === true ? {} : nuxt.options.vite, {
        optimizeDeps: {
          include: ["html-tags", "base64-js"]
        }
      });
    }
    options.sources.forEach((source) => {
      if (typeof source === "string") {
        nuxt.options.ignore.push(join("content", "**"));
      }
    });
    addPlugin(resolveRuntimeModule("./plugin"));
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.handlers = nitroConfig.handlers || [];
      nitroConfig.handlers.push({
        method: "get",
        route: `/api/${options.base}/query/:params`,
        handler: resolveRuntimeModule("./server/api/query")
      });
      nitroConfig.handlers.push({
        method: "get",
        route: `/api/${options.base}/cache`,
        handler: resolveRuntimeModule("./server/api/cache")
      });
      if (!nuxt.options.dev) {
        nitroConfig.prerender.routes.push("/api/_content/cache");
      }
      const sources = useContentMounts(nuxt, contentContext.sources || []);
      nitroConfig.storage = Object.assign(nitroConfig.storage || {}, sources);
      const localSources = Object.entries(sources).filter(([_, source]) => source.driver === "fs");
      nitroConfig.serverAssets = nitroConfig.serverAssets || [];
      nitroConfig.serverAssets.push(...localSources.map(([baseName, source]) => ({
        baseName,
        dir: source.base
      })));
      nitroConfig.externals = defu(typeof nitroConfig.externals === "object" ? nitroConfig.externals : {}, {
        inline: [
          resolve("./runtime")
        ]
      });
      nitroConfig.autoImport = nitroConfig.autoImport || {};
      nitroConfig.autoImport.imports = nitroConfig.autoImport.imports || [];
      nitroConfig.autoImport.imports.push(...[
        { name: "parse", as: "contentParse", from: resolveRuntimeModule("./server/transformers") },
        { name: "transform", as: "contentTransform", from: resolveRuntimeModule("./server/transformers") }
      ]);
      nitroConfig.virtual = nitroConfig.virtual || {};
      nitroConfig.virtual["#content/virtual/transformers"] = [
        templateUtils.importSources(contentContext.transformers),
        `const transformers = [${contentContext.transformers.map(templateUtils.importName).join(", ")}]`,
        "export const getParser = (ext) => transformers.find(p => p.extentions.includes(ext) && p.parse)",
        'export const getTransformers = (ext) => transformers.filter(p => ext.match(new RegExp(p.extentions.join("|"))) && p.transform)',
        "export default () => {}"
      ].join("\n");
    });
    addAutoImport([
      { name: "queryContent", as: "queryContent", from: resolveRuntimeModule("./composables/query") },
      { name: "highlightCode", as: "highlightCode", from: resolveRuntimeModule("./composables/highlight") },
      { name: "withContentBase", as: "withContentBase", from: resolveRuntimeModule("./composables/utils") },
      { name: "useUnwrap", as: "useUnwrap", from: resolveRuntimeModule("./composables/utils") }
    ]);
    addComponentsDir({
      path: resolve("./runtime/components"),
      pathPrefix: false,
      prefix: "",
      level: 999,
      global: true
    });
    if (options.navigation) {
      addAutoImport({ name: "fetchContentNavigation", as: "fetchContentNavigation", from: resolveRuntimeModule("./composables/navigation") });
      nuxt.hook("nitro:config", (nitroConfig) => {
        nitroConfig.handlers.push({
          method: "get",
          route: `/api/${options.base}/navigation/:params`,
          handler: resolveRuntimeModule("./server/api/navigation")
        });
      });
    }
    if (options.highlight) {
      contentContext.transformers.push(resolveRuntimeModule("./server/transformers/shiki"));
      nuxt.hook("nitro:config", (nitroConfig) => {
        nitroConfig.handlers.push({
          method: "get",
          route: `/api/${options.base}/highlight/:params`,
          handler: resolveRuntimeModule("./server/api/highlight")
        });
      });
    }
    await nuxt.callHook("content:context", contentContext);
    contentContext.defaultLocale = contentContext.defaultLocale || contentContext.locales[0];
    contentContext.markdown = processMarkdownOptions(nuxt, contentContext.markdown);
    nuxt.options.runtimeConfig.public.content = {
      base: options.base,
      tags: contentContext.markdown.tags,
      highlight: options.highlight,
      wsUrl: ""
    };
    nuxt.options.runtimeConfig.content = contentContext;
    if (!nuxt.options.dev || !options.watch) {
      return;
    }
    const storage = createStorage();
    const mounts = useContentMounts(nuxt, contentContext.sources || []);
    for (const mount in mounts) {
      storage.mount(mount, getMountDriver(mounts[mount]));
    }
    const ws = createWebSocket();
    nuxt.hook("close", async () => {
      await Promise.all([
        storage.dispose(),
        ws.close()
      ]);
    });
    const { server, url } = await listen(() => "Nuxt Content", { port: 4e3, showURL: false });
    server.on("upgrade", ws.serve);
    nuxt.options.runtimeConfig.public.content.wsUrl = url.replace("http", "ws");
    const broadcast = debounce((event, key) => {
      key = key.substring(MOUNT_PREFIX.length);
      logger.info(`${key} ${event}d`);
      ws.broadcast({ event, key });
    }, 50);
    storage.watch(broadcast);
  }
});

export { module as default };
